/**********************************************************
 Projet     :	agitator_heater
 Fichier(s) :	agit_v01.c
 Objet      : 	agitateur
 Cible : Atmega328
 frequ : 16MHz

 Derniere modification : 10/04/2013
**********************************************************/
// Fichiers  inclure :
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include "lcd.h"

// Equivalences :
#define BIT0 0b00000001
#define BIT1 0b00000010
#define BIT2 0b00000100
#define BIT3 0b00001000
#define BIT4 0b00010000
#define BIT5 0b00100000
#define BIT6 0b01000000
#define BIT7 0b10000000

//function prototypes :
void ADC_GO (char);
void out_pwm(unsigned int);
void load_preset(unsigned int);
void save_preset(unsigned int);
void load_parameters(void);
void save_parameters(void);
signed int ntc (unsigned int);

unsigned int aquis_rot(unsigned long, unsigned long, unsigned long, unsigned long);
char push_btrot(void);
char push_btrot_long(void);
char push_bt1(void);
char push_bt2(void);

int lcd_putchar(char c, FILE *stream);
int uart_putchar(char c, FILE *stream);
FILE lcd = FDEV_SETUP_STREAM(lcd_putchar, NULL, _FDEV_SETUP_WRITE);
FILE uart = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);


// constantes :

// definition des entres :
#define rot1		((PINC & BIT2) >> 2)
#define rot2		((PINC & BIT3) >> 3)
#define btrot		((PINC & BIT4) >> 4)
#define bt1 		((PINC & BIT5) >> 5)
#define bt2 		((PIND & BIT2) >> 2)
#define step		((PINB & BIT3) >> 3)

/*Affectation des voies :
PB0 : out	LCD_E
PB1 : out	PWM
PB2 : out	stepper_dir
PB3 : out	stepper_step / ISP_MOSI
PB4 : out	LCD_RS / ISP_MISO
PB5 : out	LCD_RW /ISP_SCK
PB6 : in	quartz
PB7 : in	quartz

PC0 : in	NTC1
PC1 : in	NTC2
PC2 : in	rot1
PC3 : in	rot2
PC4 : in	push_rot
PC5 : in	bt_screen
PC6 : in	ISP_RESET

PD0 : in	RX
PD1 : out	TX
PD2 : in	bt_start
PD3 : in	stepper_E
PD4 : out	LCD_bit4
PD5 : out	LCD_bit5
PD6 : out	LCD_bit6
PD7 : out	LCD_bit7
*/

//variables boutons
volatile unsigned char 	buff_rot1, buff_rot2, buff_btrot;
volatile unsigned char 	buff_bt1, buff_bt2;
volatile unsigned int	cpt_btrot;
volatile unsigned char	flag_push_btrot_long2;

//variables globales :
volatile unsigned char	refresh, char_off, init;
unsigned char			lcd_adress, i;	
unsigned char			mode, param;
unsigned char			no_preset;
unsigned char			flag_save_preset, flag_save_parameters;

volatile unsigned long	timer_set, timer_run, chrono;
unsigned char			timer_h, timer_m, timer_s;
unsigned char			timer_set_h, timer_set_m, timer_set_s;

unsigned char			div_adc, div_pid;
unsigned int			div_uart, div_aff, div_chrono, cpt_reg_t2;
volatile unsigned char	ok_adc, ok_uart, update_pwm;

volatile signed int		t1_set, t1_set2, t2_set;
volatile unsigned char	agit_speed, agit_speed_p, div_rpm, cpt_timer2, div_timer2, cpt_startup;
volatile unsigned int	agit_rpm;

unsigned long			pwm_100;
unsigned char			run, state;

//PID
volatile unsigned char 	t1_max, kp, ki, kd;
volatile signed long	err, sum, prop, integ, deriv, pwm_out;
unsigned int			fifo[10];

//ADC
unsigned int			mesure;
unsigned int			moyreg;
unsigned char			nbrconv;
unsigned int			analog_value;
volatile unsigned char	analog_in;
volatile signed int		t1, t2;

//custom char definition
static const PROGMEM unsigned char custchar[24] =
{4,14,21,4,4,4,4,0,
0,4,4,4,4,21,14,4,
8,20,8,3,4,4,4,3};

//NTC LUT : ntc = 10k @ 25C, pull-up = 2.4k
signed int ntc_a[16] =
{-185, -185, -185, -185, -158, -133, -120, -111,
-106, -107, -109, -119, -136, -167, -246, -472};

signed int ntc_b[16] =
{1417, 1417, 1417, 1417, 1347, 1269, 1218, 1178,
1153, 1157, 1169, 1236, 1367, 1626, 2338, 4513};

// ==============================   ZONE  PROGRAMME ==============================

int main(void)
{	
	//Config portB
	DDRB=0b00111111;				//PB0-5 en sortie
	PORTB=0x00;
	
	//Config portC
	DDRC=0b00000000;				//PORTC en entre
	PORTC=0b00111100;				//pull-up sur PC2-5
	
	//Config portD
	DDRD=0b11111010;				//PD1, PD3-7 en sortie
	PORTD=0b00000100;				//pull-up sur PD2
	
	//Config Timer0
	TCCR0A=0b00000010;				//Mode CTC
	TCCR0B=0b00000011;				//clk=fquartz/64
	OCR0A =249;						//f=1kHz
	
	//Config timer1
	TCCR1A=0b00000010;				//fast PWM top=ICR1 outputs:off
	TCCR1B=0b00011011;   			//clk timer1 = clk/64
	ICR1  =2499;					//f=100Hz
	
	//config TIMER2
	TCCR2A=0b00000010;				//Mode CTC
	TCCR2B=0b00000011;				//Fclk_timer1 = Fquartz/32
	OCR2A =233;						//Ftimer2 = 2137Hz = ~1000RPM
	
	//Config ADC
	ADMUX=0b00000000;				//reference=Aref, ajustement  droite
	ADCSRA=0b10001111;				//CAN ON, interrupt ON, prescaler  128 (62,5kHz)
	
	//config UART
	//UBRRL=8;	UBRRH=0;			//baud rate = 115200 bauds
	UBRR0L=103;	UBRR0H=0;			//baud rate = 9600 bauds
	UCSR0A=0b00000000;				//Double baud rate = OFF
	UCSR0B=0b00001000;				//mission active
	UCSR0C=0b00000110;				//Mode asynchrone, pas de parit, 1bit de stop, caractere de 8bit
	
	//Autorisation interruptions
	TIMSK0=0b0000010;				//Autorisation interruption T0_COMPAREA
	TIMSK2=0b0000010;				//Autorisation interruption T2_COMPAREA
	sei (); 						//Valide toutes les interruptions
	
	// enable watchdog timer : 1s
	wdt_reset(); // reset watchdog
	wdt_enable(WDTO_1S);
	
	//LCD init & load custom char :
	lcd_init(LCD_DISP_ON);
	lcd_command(_BV(LCD_CGRAM));	//set CG RAM start address 0
	for(lcd_adress=0; lcd_adress<24; lcd_adress++)
		lcd_data(pgm_read_byte_near(&custchar[lcd_adress]));
	
	//message serie
	fprintf_P(&uart, PSTR("\rtest serie\r"));
	
	while (1)
	{	
		wdt_reset(); // reset watchdog
		
		if(init == 0)
		{		
			//initialisation boutons :
			buff_rot1 = rot1;
			buff_rot2 = rot2;
			buff_btrot = btrot;
			buff_bt1 = bt1;
			buff_bt2 = bt2;
			aquis_rot(0,0,0,0);					//initialize rotary switch
			load_parameters();
			
			mode		= 0;
			refresh		= 1;
			no_preset	= 1;
			init		= 1;
			flag_save_preset = 0;
			flag_save_parameters = 0;
			
			PORTB &= ~BIT2;			//stepper dir = 0
		}
		
		switch (mode)
		{	
			case 0:		//menu principal
				no_preset = aquis_rot(no_preset, 1, 1, 4);
				if(refresh)
				{
					refresh = 0;
					switch (no_preset)
					{
						case 1:		//sel=0
							lcd_gotoxy(0,0);					//dbut de la 1e ligne 
							fprintf_P(&lcd, PSTR(">remove PLA<    "));
							lcd_gotoxy(0,1);					//dbut de la 2e ligne 
							fprintf_P(&lcd, PSTR(" remove PVA    "));
							
							lcd_gotoxy(15,1);					
							lcd_putc(1);						//fleche bas
						break;
						
						case 2:		//sel=1
							lcd_gotoxy(0,0);					//dbut de la 1e ligne 
							fprintf_P(&lcd, PSTR(">remove PVA<   "));
							lcd_gotoxy(0,1);					//dbut de la 2e ligne 
							fprintf_P(&lcd, PSTR(" smooth ABS    "));
							
							lcd_gotoxy(15,0);
							lcd_putc(0);						//fleche haut
							lcd_gotoxy(15,1);
							lcd_putc(1);						//fleche bas
						break;
						
						case 3:		//sel=2
							lcd_gotoxy(0,0);					//dbut de la 1e ligne 
							fprintf_P(&lcd, PSTR(">smooth ABS<   "));
							lcd_gotoxy(0,1);					//dbut de la 2e ligne
							fprintf_P(&lcd, PSTR(" other         "));
							
							lcd_gotoxy(15,0);
							lcd_putc(0);						//fleche haut
							lcd_gotoxy(15,1);
							lcd_putc(1);						//fleche bas
						break;
						
						case 4:		//sel=3
							lcd_gotoxy(0,0);					//dbut de la 1e ligne 
							fprintf_P(&lcd, PSTR(">other<        "));
							lcd_gotoxy(0,1);					//dbut de la 2e ligne
							fprintf_P(&lcd, PSTR("                "));
							
							lcd_gotoxy(15,0);
							lcd_putc(0);						//fleche haut
						break;
						
						default:
						break;
					}
				}
				
				//select preset
				if(push_btrot() || push_bt1())
				{
					load_preset(no_preset*8);
					mode = 1;
					lcd_clrscr();
				}
			break;
			
			case 1:		//settings
				if(refresh)
				{
					if(char_off == 0)
					{
						lcd_gotoxy(0,0);					//1st character, 1st line
						fprintf_P(&lcd, PSTR("T1s:%3u"), t1_set);
						lcd_putc(2);
						fprintf_P(&lcd, PSTR(" T2s:%2u"), t2_set);
						lcd_putc(2);
						
						lcd_gotoxy(0,1);					//1st character, 2nd line
						fprintf_P(&lcd, PSTR("%.2u:%.2u:%.2u"), timer_set_h, timer_set_m, timer_set_s);
						
						refresh = 0;
					}
					lcd_gotoxy(13,1);					//13th character, 2nd line
					if(flag_save_preset == 0)
						fprintf_P(&lcd, PSTR("   "));
					else
						fprintf_P(&lcd, PSTR("sav"));
				}
				
				switch (param)
				{
					case 0:			//changement consigne t1;
						t1_set = aquis_rot(t1_set, 1, 0, t1_max);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(4,0);						//pointer sur caractere  ffacer
							fprintf_P(&lcd, PSTR("   "));			//eteindre les caractres
							refresh = 0;
						}
					break;
					
					case 1:			//changement consigne t2;
						t2_set = aquis_rot(t2_set, 1, 0, 99);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(13,0);						//pointer sur caractere  ffacer
							fprintf_P(&lcd, PSTR("   "));			//eteindre les caractres
							refresh = 0;
						}
					break;
					
					case 2:			//changement consigne timer_h
						timer_set_h = aquis_rot(timer_set_h, 1, 0, 24);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(0,1);	 					//pointer sur caractere a ffacer
							fprintf_P(&lcd, PSTR("  "));			//eteindre caracteres
							refresh = 0;
						}
					break;
					
					case 3:			//changement consigne timer_m
						timer_set_m = aquis_rot(timer_set_m, 1, 0, 59);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(3,1);	 					//pointer sur caractere a ffacer
							fprintf_P(&lcd, PSTR("  "));			//eteindre caracteres
							refresh = 0;
						}
					break;
					
					case 4:			//changement consigne timer_s
						timer_set_s = aquis_rot(timer_set_s, 1, 0, 59);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(6,1);	 					//pointer sur caractere a ffacer
							fprintf_P(&lcd, PSTR("  "));			//eteindre caracteres
							refresh = 0;
						}
					break;
				}
				
				//selection parametre  modifier suiv
				if(push_btrot())
				{
					if(param == 4)
						param = 0;
					else
						param ++;
					
					flag_save_preset = 0;
				}
				
				//next screen
				if(push_bt1())
				{
					timer_set = timer_set_h*3600 + timer_set_m*60 + timer_set_s;
					flag_save_preset = 0;
					param = 0;
					mode = 2;
					lcd_clrscr();
				}
				
				//save preset
				if(push_btrot_long() && flag_save_preset == 0)
				{
					save_preset(no_preset*8);
					flag_save_preset = 1;
				}
			break;
			
			case 2:		//agitator
				if(refresh)
				{	
					lcd_gotoxy(0,0);					//1st character, 1st line
					fprintf_P(&lcd, PSTR("Stirring :"));
					
					if(char_off == 0)
					{
						lcd_gotoxy(0,1);					//12th character, 1st line
						if(agit_speed > 0)
							fprintf_P(&lcd, PSTR("%4uRPM"),agit_rpm);
						else
							fprintf_P(&lcd, PSTR("OFF    "));
						
						refresh = 0;
					}
					
					lcd_gotoxy(13,1);					//13th character, 2nd line
						if(flag_save_preset == 0)
							fprintf_P(&lcd, PSTR("   "));
						else
							fprintf_P(&lcd, PSTR("sav"));
				}
				
				agit_speed = aquis_rot(agit_speed, 1, 0, 4);
				if(refresh && char_off)
				{
					lcd_gotoxy(0,1);						//1st character, 2nd line
					fprintf_P(&lcd, PSTR("    "));			//erase characters
					refresh = 0;
				}
				
				if(agit_speed != agit_speed_p)
				{
					if(agit_speed > 0)
					{
						div_rpm = 1 << (4 - agit_speed);
						agit_rpm = 1000 / div_rpm;
						
						div_timer2 = 10;
						cpt_timer2 = 0;
					}
					flag_save_preset = 0;
				}
				agit_speed_p = agit_speed;
				
				//next screen
				if(push_btrot() || push_bt1())
				{
					flag_save_preset = 0;
					param = 0;
					mode = 3;
					lcd_clrscr();
				}
				
				//save preset
				if(push_btrot_long() && flag_save_preset == 0)
				{
					save_preset(no_preset*8);
					flag_save_preset = 1;
				}
			break;
			
			case 3:		//temp
				if(refresh)
				{
					lcd_gotoxy(0,0);					//1st character, 1st line
					fprintf_P(&lcd, PSTR("T1:%3d"), t1);
					lcd_putc(2);
					
					if(t2>0)
					{
						fprintf_P(&lcd, PSTR("   T2:%2d"), t2);
						lcd_putc(2);
					}
					else
						fprintf_P(&lcd, PSTR("   T2:NC "));
					
					lcd_gotoxy(0,1);					//1st character, 2nd line
					timer_h = timer_run/3600;
					timer_m = (timer_run - (timer_h*3600))/60;
					timer_s = timer_run - (timer_h*3600) - (timer_m*60);
					fprintf_P(&lcd, PSTR("%.2u:%.2u:%.2u"), timer_h, timer_m, timer_s);
					
					lcd_gotoxy(13,1);					//13th character, 2nd line
					switch (state)
					{
						case 0:	
							fprintf_P(&lcd, PSTR("RDY"));
						break;
						
						case 1:	
							fprintf_P(&lcd, PSTR("RUN"));
						break;
						
						case 2:
							fprintf_P(&lcd, PSTR("END"));
						break;
						
						case 3:
							fprintf_P(&lcd, PSTR("ERR"));
						break;
					}
					refresh = 0;
				}
				
				//next screen
				if(push_bt1())
				{
					if(btrot == 0)
					{
						param = 0;
						mode = 4;
						lcd_clrscr();
					}
					else
					{
						param = 0;
						mode = 0;
						lcd_clrscr();
					}
				}
			break;
			
			case 4:		//parametters mode 1
				if(refresh && char_off == 0)
				{	
					lcd_gotoxy(0,0);					//1st character, 1st line
					fprintf_P(&lcd, PSTR("T1max:%.3u"), t1_max);
					lcd_putc(2);						//C
					
					lcd_gotoxy(13,0);					//12th character, 1st line
					if(flag_save_parameters)
						fprintf_P(&lcd, PSTR("sav"));
					else
						fprintf_P(&lcd, PSTR("   "));
					
					lcd_gotoxy(0,1);					//1st character, 2nd line
					fprintf_P(&lcd, PSTR("P:%.2u  I:%.2u  D:%.2u"), kp, ki, kd);
					
					refresh = 0;
				}
				
				switch (param)
				{
					case 0:		//T1max
						t1_max = aquis_rot(t1_max, 1, 0, 150);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(6,0);				//6th character, 1st line
							fprintf_P(&lcd, PSTR("   "));	//erase characters
							refresh = 0;
						}
					break;
					
					case 1:		//T1 kP
						kp = aquis_rot(kp, 1, 0, 99);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(2,1);				//3th character, 2nd line
							fprintf_P(&lcd, PSTR("  "));	//erase characters
							refresh = 0;
						}
					break;
					
					case 2:		//T1 kI
						ki = aquis_rot(ki, 1, 0, 99);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(8,1);				//9th character, 2nd line
							fprintf_P(&lcd, PSTR("  "));	//erase characters
							refresh = 0;
						}
					break;
					
					case 3:		//T1 kD
						kd = aquis_rot(kd, 1, 0, 99);
						
						if(refresh && char_off)
						{
							lcd_gotoxy(14,1);				//15th character, 2nd line
							fprintf_P(&lcd, PSTR("  "));	//erase characters
							refresh = 0;
						}
					break;
				}
				
				if(push_btrot())
				{
					if(param == 3)
						param = 0;
					else
						param++;
					
					flag_save_parameters = 0;
				}
				
				//save
				if(push_btrot_long() && flag_save_parameters == 0)
				{
					save_parameters();
					flag_save_parameters = 1;
				}
				
				//next screen
				if(push_bt1())
				{
					flag_save_parameters = 0;
					param = 0;
					mode = 0;
					lcd_clrscr();
				}
			break;
		}	//end switch mode
		
		//start
		if(push_bt2() && mode == 3)
		{
			t1_set2 = t1_set;
			div_timer2 = 10;
			div_chrono = 0;
			chrono = 0;
			integ = 0;
			run = 1;
			state = 1;	//RUN
			
			//serial
			//fprintf_P(&uart, PSTR("\rTime(s)	T1_set(c)	T1(c)	T2_set(c)	T2(c)	pwm(%%)"));
			fprintf_P(&uart, PSTR("\rprop	integ	deriv	pwm	t1_set2"));
			div_uart = 0;
			ok_uart = 1;
		}
		
		//manual stop/reset
		if(bt2 == 1)
		{
			run = 0;
			state = 0;	//RDY
			out_pwm(0);
			timer_run = timer_set;
			refresh = 1;
		}
		
		//auto stop
		if( run && timer_run == 0)
		{
			run = 0;
			state = 2;	//END
			out_pwm(0);
			refresh = 1;
		}
		
		//error
		if(run && (t1 > t1_max || t1 < 0))
		{
			run = 0;
			state = 3;	//ERR
			out_pwm(0);
			refresh = 1;
			pwm_100 = ((long)pwm_out*100)/2499;
			fprintf_P(&uart, PSTR("\r%5lu	%3d	%3d	%3d	%3d	%3lu	ERROR"), chrono, t1_set, t1, t2_set, t2, pwm_100);
		}
		//PWM management
		if(update_pwm)
		{
			update_pwm = 0;
			
			if(run)
			{
				//proportionnal
				prop = ((long)t1_set2 - t1)*64*kp;
				
				//integral
				if(t1 < t1_set2)
				{
					err=((long)t1_set2 - t1)*ki;
					
					if(integ + err < 25000)
						integ += err;
					else
						integ = 25000;
				}
				else if (t1 > t1_set2)
				{
					err=((long)t1 - t1_set2)*ki;
					
					if(integ - err > -25000)
						integ -= err;
					else
						integ =- 25000;
				}
				
				//derivative
				deriv = ((long)fifo[0]-t1)*kd;
				
				for(i=0;i<9;i++)
				{
					fifo[i] = fifo[i+1];
				}
				fifo[9] = t1;
				
				pwm_out = (prop + integ + deriv)/10;
				if(pwm_out > 2499)
					pwm_out = 2499;
				else if (pwm_out < 0)
					pwm_out = 0;
				
				//regulation T2
				cpt_reg_t2++;
				if(cpt_reg_t2 == 100)	//every 10s
				{
					cpt_reg_t2 = 0;
					if(t2 > t2_set && t1_set2 > 0)
						t1_set2--;
					else if(t2 < t2_set && t1_set2 < t1_set)
						t1_set2++;
				}
			}
			else
				pwm_out = 0;
			
			out_pwm(pwm_out);
		}
		
		//Message srie
		if(ok_uart)
		{
			ok_uart = 0;
			if(run)
			{
				pwm_100 = ((long)pwm_out*100)/2499;
				//fprintf_P(&uart, PSTR("\r%5lu	%3d	%3d	%3d	%3d	%3u"), chrono, t1_set, t1, t2_set, t2, pwm_100);
				fprintf_P(&uart, PSTR("\r%5ld	%5ld	%5ld	%5ld	%3d"), prop, integ, deriv, pwm_out, t1_set2);
			}
		}
	} // end while
}	//end main

//analog/digital converter
ISR(ADC_vect)
{
	mesure = ADCL;
	mesure += (ADCH<<8);
	moyreg += mesure;
	nbrconv++;
	if(nbrconv > 3)
	{
		nbrconv = 0;
		analog_value = moyreg/4;
		moyreg = 0;
		
		switch (analog_in)
		{	
			case 0 :	//NTC 1
				t1 = ntc(analog_value);
				analog_in = 1;
			break;
			
			case 1 :	//NTC2
				t2 = ntc(analog_value);
				analog_in = 0;
				ok_adc = 0;							//end of conversion
				refresh = 1;
				update_pwm = 1;
			break;
		}
	}
	if(ok_adc)
		ADC_GO (analog_in);
}

//Timer 0 compare
ISR(TIMER0_COMPA_vect)
{
	//blinking timer
	div_aff++;
	if (div_aff == 250)		//character off time = 250ms
	{
		char_off = 0;
		refresh=1;
	}
	if (div_aff >= 1000)	//blinking freq = 1Hz
	{
		div_aff = 0;
		char_off = 1;
		refresh=1;
	}

	//UART
	div_uart++;
	if(div_uart == 1000)	//1Hz
	{
		div_uart = 0;
		ok_uart = 1;
	}
	
	//ADC
	div_adc++;
	if(div_adc == 100)		//10Hz
	{
		div_adc = 0;
		ok_adc = 1;
		ADC_GO (0);
	}
	
	//RTC
	div_chrono++;
	if(div_chrono == 1000)	//1Hz
	{
		div_chrono = 0;
		if(run == 1 && timer_run > 0)
		{
			timer_run--;
			chrono ++;
		}
	}
	
	//cpt push rot long
	if(btrot == 0)
		cpt_btrot ++;
	else
		cpt_btrot = 0;
}

//Interruption Timer2 OVF
ISR(TIMER2_COMPA_vect)
{
	if(run && agit_speed > 0)
	{
		//startup ramp
		cpt_startup++;
		if(cpt_startup >= 128)	//2 tour @1000rpm
		{
			cpt_startup = 0;
			if(div_timer2 > div_rpm)
				div_timer2 --;
		}
		
		//step
		cpt_timer2++;
		if(cpt_timer2 >= div_timer2)
		{
			cpt_timer2 = 0;
			PORTD |= BIT3;			//sortie stepper_enable  1
			if(step)
				PORTB &= ~BIT3;			//sortie PB3  0
			else
				PORTB |= BIT3;			//sortie PB3  1
		}
	}
	else
	{
		PORTD &= ~BIT3;			//sortie stepper_enable  0
		PORTB &= ~BIT3;			//sortie step  0
	}
}

//encoder
unsigned int aquis_rot(unsigned long nbr, unsigned long inc, unsigned long min, unsigned long max)
{
	//gestion rotatif (rot2 = sync, rot1 = sens de rotation)	
	if(buff_rot2 == 1 && rot2 == 0)	//front desecendant sur rot2
	{
		buff_rot1 = rot1;
		//_delay_ms(6);
		if(rot2 == 0)
		{
			if (buff_rot1)
			{
				if(nbr > (min + inc))
					nbr = nbr-inc;
				else
					nbr = min;
			}
			else
			{
				if(nbr < (max-inc))
					nbr = nbr+inc;
				else
					nbr = max;
			}
			refresh = 1;
		}
	}
	/*
	else if(buff_rot2 == 0 && rot2 == 1)	//front montant sur rot2
	{
		buff_rot1 = rot1;
		_delay_ms(6);
		if(rot2 == 1)
		{
			if(buff_rot1)
			{	
				if(nbr < (max-inc))
					nbr = nbr+inc;
				else
					nbr = max;
			}
			else
			{
				if(nbr > (min + inc))
					nbr = nbr-inc;
				else
					nbr = min;
			}
			refresh = 1;
		}
	}*/
	buff_rot2 = rot2;
	return nbr;
}

//push encoder
char push_btrot(void)
{
	unsigned char flag_btrot = 0;
	
	if(buff_btrot == 0 && btrot == 1)	//front montant sur push rot
	{
		_delay_ms(10);
		if(btrot == 1 && flag_push_btrot_long2 == 0)
			flag_btrot = 1;
		
		flag_push_btrot_long2 = 0;
		cpt_btrot = 0;
		refresh = 1;
	}
	buff_btrot = btrot;
	return flag_btrot;
}

//push encoder long
char push_btrot_long(void)
{
	unsigned char flag_push_btrot_long = 0;

	if(cpt_btrot > 2000 && flag_push_btrot_long2 == 0)	
	{
		flag_push_btrot_long = 1;
		flag_push_btrot_long2 = 1;
		refresh = 1;
	}
	
	if(buff_btrot == 0 && btrot == 1)	//relachement de push rot
	{
		flag_push_btrot_long2 = 0;
		cpt_btrot = 0;
	}
	buff_btrot = btrot;
	return flag_push_btrot_long;
}

//button 1
char push_bt1(void)
{
	unsigned char flag_bt1 = 0;
	
	if(buff_bt1 == 1 && bt1 == 0)	//falling edge on button 1 input
	{
		_delay_ms(10);
		if(bt1 == 0)
			flag_bt1 = 1;
		
		refresh = 1;
	}
	
	buff_bt1 = bt1;
	return flag_bt1;
}

//button 2
char push_bt2(void)
{
	unsigned char flag_bt2 = 0;
	
	if(buff_bt2 == 1 && bt2 == 0)	//falling edge on button 2 input
	{
		_delay_ms(10);
		if(bt2 == 0)
			flag_bt2 = 1;
		
		refresh = 1;
	}
	
	buff_bt2 = bt2;
	return flag_bt2;
}

//load temperature preset values
void load_preset(unsigned int start_adress)
{
	t1_set = eeprom_read_byte((uint8_t*)start_adress);
	if(t1_set > 200)
		t1_set = 100;

	t2_set = eeprom_read_byte((uint8_t*)(start_adress+1));
	if(t2_set > 100)
		t2_set = 60;
		
	timer_set_h = eeprom_read_byte((uint8_t*)(start_adress+2));
	if(timer_set_h > 24)
		timer_set_h = 0;
	
	timer_set_m = eeprom_read_byte((uint8_t*)(start_adress+3));
	if(timer_set_m > 59)
		timer_set_m = 0;
	
	timer_set_s = eeprom_read_byte((uint8_t*)(start_adress+4));
	if(timer_set_s > 59)
		timer_set_s = 0;

	agit_speed = eeprom_read_byte((uint8_t*)(start_adress+5));
	if(agit_speed > 4)
		agit_speed = 0;
}

//Save preset
void save_preset(unsigned int start_adress)
{
	eeprom_update_byte((uint8_t*)start_adress,t1_set);
	eeprom_update_byte((uint8_t*)(start_adress+1),t2_set);
	eeprom_update_byte((uint8_t*)(start_adress+2),timer_set_h);
	eeprom_update_byte((uint8_t*)(start_adress+3),timer_set_m);
	eeprom_update_byte((uint8_t*)(start_adress+4),timer_set_s);
	eeprom_update_byte((uint8_t*)(start_adress+5),agit_speed);
}

//load PID parameters
void load_parameters(void)
{
	//ext1
	t1_max = eeprom_read_byte((uint8_t*)1);
	if(t1_max > 200)
		t1_max = 100;
	
	kp = eeprom_read_byte((uint8_t*)3);
	if(kp > 99)
		kp = 0;
		
	ki = eeprom_read_byte((uint8_t*)4);
	if(ki > 99)
		ki = 0;
	
	kd = eeprom_read_byte((uint8_t*)5);
	if(kd > 99)
		kd = 0;
}

void save_parameters(void)
{
	eeprom_update_word((uint16_t*)1,t1_max);
	eeprom_update_byte((uint8_t*)3,kp);
	eeprom_update_byte((uint8_t*)4,ki);
	eeprom_update_byte((uint8_t*)5,kd);
}

signed int ntc (unsigned int conv)
{
	signed long 	temp;
	unsigned char 	i;
	
	i = conv / 64;
	temp = conv;
	temp *= ntc_a[i];
	temp += (long) ntc_b[i]*100;
	temp /= 1000;
	return temp;
}

void out_pwm(unsigned int cons)
{
	if(cons == 0)
	{
		TCCR1A &= 0b00111111;	//pwm output OCR1A disconnected
		PORTB &= ~BIT1;			//portB.1  0
	}
	else
	{
		TCCR1A |= 0b10000000;	//pwm output OCR1A connected
		OCR1A= cons;
	}
}

//Lancement d'une conversion analogique/numerique
void ADC_GO (char port)
{
	port &= 0x0F;				//masque port entre 0 et 15
	ADMUX &= 0xF0;				//RAZ de MUX0  MUX3
	ADMUX |= port;				//slection du port
	ADCSRA |= 0b01000000;		//lancer une conversion
}

//Send char on serial out
int uart_putchar(char c, FILE *stream)
{
    while ((UCSR0A & (1<<UDRE0))==0);
    UDR0 = c;
    return 0;
}

//Fait le lien entre printf et le LCD
int lcd_putchar(char c, FILE *stream) 
{ 
   lcd_putc(c); 
   return 0; 
}
